package core;

public class Rectangle {

	private Point topLeft;
	private int width;
	private int height;

	// getters
	public int getWidth() {
		return width;
	}

	public int getHeight() {
		return height;
	}

	public int getTop() {
		return topLeft.getY();
	}

	public int getLeft() {
		return topLeft.getX();
	}

	public int getRight() {
		return getLeft() + getWidth();
	}

	public int getBottom() {
		return getTop() + getHeight();
	}

	// setters

	// operations
	public void translate(int x, int y) {
		topLeft.translate(x, y);
	}

	public void inflate(int dx, int dy) {
		width += dx;
		height += dy;
	}

	public void unite(Rectangle other) {
		int left = Math.min(getLeft(), other.getLeft());
		int right = Math.max(getRight(), other.getRight());
		int top = Math.min(getTop(), other.getTop());
		int bottom = Math.max(getBottom(), other.getBottom());

		moveTo(left, top);
		setDimensions(right - left, bottom - top);
	}

	public void moveTo(int x, int y) {
		topLeft.moveTo(x, y);
	}

	public void setDimensions(int width, int height) {
		this.width = width;
		this.height = height;
	}

	public boolean isContained(Point p) {
		return isContained(p.getX(), p.getY());
	}

	public boolean isContained(int x, int y) {
		return (x <= getRight()) && (x >= getLeft()) && (y <= getBottom())
				&& (y >= getTop());
	}

	public boolean isIntersecting(Rectangle r) {
		return isContained(r.getLeft(), r.getTop())
				|| isContained(r.getRight(), r.getTop())
				|| isContained(r.getLeft(), r.getBottom())
				|| isContained(r.getRight(), r.getBottom());
	}

	// constructor
	public Rectangle(int x, int y, int width, int height) {
		topLeft = new Point(x, y);
		this.width = width;
		this.height = height;
	}

	public Rectangle(int width, int height) {
		topLeft = new Point(0, 0);
		this.width = width;
		this.height = height;
	}

	public Rectangle(Rectangle copy) {
		topLeft = new Point(copy.topLeft);
		this.width = copy.width;
		this.height = copy.height;
	}
}
