package model;

public class Color {
	// Predefined colors (for convenience)
	public static final Color Transparent = new Color(0, 0, 0, 0);
	public static final Color Black = new Color(0, 0, 0, 255);
	public static final Color White = new Color(255, 255, 255, 255);
	public static final Color Red = new Color(255, 0, 0, 255);
	public static final Color Green = new Color(0, 255, 0, 255);
	public static final Color Blue = new Color(0, 0, 255, 255);

	private int red;
	private int green;
	private int blue;
	private int alpha;

	/*
	 * @pre 0 <= red <= 255
	 * 
	 * @pre 0 <= green <= 255
	 * 
	 * @pre 0 <= blue <= 255
	 * 
	 * @pre 0 <= alpha <= 255
	 * 
	 * @post getRed() == red
	 * 
	 * @post getGreen() == green
	 * 
	 * @post getBlue() == blue
	 * 
	 * @post getAlpha() == alpha
	 */
	public Color(int red, int green, int blue, int alpha) {
		this.red = red;
		this.green = green;
		this.blue = blue;
		this.alpha = alpha;
	}

	/**
	 * Copies the content of another color object
	 * 
	 * @param copy
	 *            The color to copy
	 */
	public Color(Color copy) {
		this(copy.red, copy.green, copy.blue, copy.alpha);
	}

	public int getRed() {
		return red;
	}

	public int getGreen() {
		return green;
	}

	public int getBlue() {
		return blue;
	}

	public int getAlpha() {
		return alpha;
	}

	public boolean isTransparent() {
		return alpha == 0;
	}
}
