package render;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.geom.Ellipse2D;

import model.CircleShape;
import model.RectangleShape;

public class GraphicDevice implements Device {

	private Graphics2D graphics;
	private Stroke lastStroke;
	private Color lastColor;
	private Paint lastPaint;

	public GraphicDevice(Graphics2D graphics) {
		this.graphics = graphics;
	}

	private void restorePaint() {
		graphics.setPaint(lastPaint);
		lastPaint = null;
	}

	private static Shape getDeviceSpecificRectangle(RectangleShape r) {
		core.Rectangle bounds = r.getBounds();
		return new Rectangle(bounds.getLeft(), bounds.getTop(),
				bounds.getWidth(), bounds.getHeight());
	}

	private static Shape getDeviceSpecificCircle(CircleShape c) {
		core.Rectangle bounds = c.getBounds();
		return new Ellipse2D.Double(bounds.getLeft(), bounds.getTop(),
				bounds.getWidth(), bounds.getHeight());
	}

	private void savePaint(model.Color fillColor) {
		lastPaint = graphics.getPaint();
		graphics.setPaint(getDeviceSpecificColor(fillColor));
	}

	public static Color getDeviceSpecificColor(model.Color color) {
		return new Color(color.getRed(), color.getGreen(), color.getBlue(),
				color.getAlpha());
	}
	
	public static model.Color getModelColor(Color c) {
		return getModelColor(c, c.getAlpha());
	}
	
	public static model.Color getModelColor(Color c, int alpha) {
		return new model.Color(c.getRed(), c.getGreen(), c.getBlue(), alpha);
	}

	private void restoreStroke() {
		graphics.setStroke(lastStroke);
		graphics.setColor(lastColor);
		lastStroke = null;
		lastColor = null;
	}

	private void saveStroke(model.Stroke stroke) {
		lastStroke = graphics.getStroke();
		lastColor = graphics.getColor();
		graphics.setStroke(new BasicStroke(stroke.getWidth()));
		graphics.setColor(getDeviceSpecificColor(stroke.getColor()));
	}

	private void drawShape(model.Shape r, Shape rect) {
		savePaint(r.getFillColor());
		graphics.fill(rect);
		restorePaint();

		saveStroke(r.getStroke());
		graphics.draw(rect);
		restoreStroke();
	}

	@Override
	public void initialize(model.Drawing document) {
		graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);

		core.Rectangle bounds = document.getBounds();
		Color deviceSpecificColor = getDeviceSpecificColor(document
				.getFillColor());
		graphics.setBackground(deviceSpecificColor);
		// graphics.clearRect(0, 0, bounds.getWidth(), bounds.getHeight());
		graphics.setColor(deviceSpecificColor);
		graphics.fillRect(0, 0, bounds.getWidth(), bounds.getHeight());
	}

	@Override
	public void draw(RectangleShape r) {
		Shape rect = getDeviceSpecificRectangle(r);

		drawShape(r, rect);
	}

	@Override
	public void draw(CircleShape c) {
		Shape circle = getDeviceSpecificCircle(c);

		drawShape(c, circle);
	}

	public static Rectangle rectangleToDeviceSpecific(core.Rectangle bounds) {
		return new Rectangle(bounds.getLeft(), bounds.getTop(), bounds.getWidth(), bounds.getHeight());
	}
}
